package pmm.antiaircraft2;

import java.util.concurrent.ArrayBlockingQueue;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.os.Handler;
import android.util.DisplayMetrics;
import android.view.SurfaceHolder;

public class GameThread extends Thread {
	private SurfaceHolder mSurfaceHolder;
	public DisplayMetrics metrics = new DisplayMetrics();
	private static Bitmap mBackgroundImage;
	public static Bitmap tankImg;
	public static Bitmap cannonImg;
	public static boolean mRun = false;
	private Tank tank;
	public static Resources res;
	private Object inputQueueMutex = new Object();
	private ArrayBlockingQueue<InputObject> inputQueue = new ArrayBlockingQueue<InputObject>(30); 

	public GameThread(SurfaceHolder surfaceHolder, Context context, Handler handler) {
		mSurfaceHolder = surfaceHolder;
		Resources res = context.getResources();
		mBackgroundImage = BitmapFactory.decodeResource(res,R.drawable.background);
		tankImg = BitmapFactory.decodeResource(res,R.drawable.tank);
		cannonImg = BitmapFactory.decodeResource(res,R.drawable.cannon);
	}

	@Override
	public void run() {
		while (mRun) {
			Canvas c = null;
			try {
				c = mSurfaceHolder.lockCanvas(null);
				synchronized (mSurfaceHolder) {
					processInput();
					draw(c);
				}
			} finally {
				if (c != null) {
					mSurfaceHolder.unlockCanvasAndPost(c);
				}
			}
		}
	}
	
	public void setRunning(boolean b) {
		mRun = b;
	}
	
	private void draw(Canvas canvas) {
		canvas.drawBitmap(mBackgroundImage, 0, 0, null);
		tank.draw(canvas); 
	}

	public void createGraphics() {
		mBackgroundImage=Bitmap.createBitmap(mBackgroundImage);
		tank = new Tank();
	}
	
	public void feedInput(InputObject input) {
		synchronized(inputQueueMutex) {
			try {
				inputQueue.put(input);
			} catch (InterruptedException e) {
			}
		}
	}
	
	private void processInput() {
		synchronized(inputQueueMutex) {
			ArrayBlockingQueue<InputObject> inputQueue = this.inputQueue;
			while (!inputQueue.isEmpty()) {
				try {
					InputObject input = inputQueue.take();
					if (input.eventType == InputObject.EVENT_TYPE_TOUCH) {
						processMotionEvent(input);
					}
					input.returnToPool();
				} catch (InterruptedException e) {
				}
			}
		}
	}
	
	private void processMotionEvent(InputObject input) {

		/**
		 * whole screen fire
		 */
		if( input.action==InputObject.ACTION_TOUCH_DOWN){

			//tank.pressFire();
			tank.setTarget(input.x, input.y);

		}	else if( input.action==InputObject.ACTION_TOUCH_MOVE) {

			tank.setTarget(input.x, input.y);
//			bullets.add(new FunctionalBullet(50, (float) (Math.PI/2), input.x, input.y));
		}

		else if( input.action==InputObject.ACTION_TOUCH_UP){

			tank.setTarget(input.x, input.y);
//			tank.releaseFire();
		}
	}
}